/**
 * Logger target for XPanel (http://www.ajaxmaker.com/xpanel/xpanel.htm)
 * @author Konstantin Kovalev aka Constantiner (constantiner@gmail.com)
 */
package constantiner.logging
{
	import flash.events.Event;
	import flash.events.IEventDispatcher;
	import flash.events.StatusEvent;
	import flash.net.LocalConnection;
	import flash.utils.getTimer;
	
	import mx.logging.AbstractTarget;
	import mx.logging.ILogger;
	import mx.logging.LogEvent;
	import mx.logging.LogEventLevel;
	import flash.events.EventDispatcher;
	
	[Event(name="fault", type="flash.events.Event")]
	
	public class XPanelTarget extends AbstractTarget implements IEventDispatcher
	{
		public static const FAULT:String = "fault";
		
		private static const LEVEL_DEBUG:int       = 0x01;
		private static const LEVEL_INFORMATION:int = 0x02;
		private static const LEVEL_WARNING:int     = 0x04;
		private static const LEVEL_ERROR:int       = 0x08;
		private static const LEVEL_NONE:int        = 0xFF;
		private static const LEVEL_ALL:int         = 0x00;
		
		private var aConnection:LocalConnection;
		
	    [Inspectable(category="General", defaultValue="false")]
	    
		/**
	     *  Indicates if the category for this target should added to the trace.
	     */
	    public var includeCategory:Boolean = false;
	    
	    [Inspectable(category="General", defaultValue=" ")]
	    
		/**
	     *  The separator string to use between fields (the default is " ")
	     */
	    public var fieldSeparator:String = " ";
    
		private var dispatcher:EventDispatcher;
		
		function XPanelTarget ()
		{
			super ();
			dispatcher = new EventDispatcher(this);
		}
		
	    /**
	     *  This method handles a <code>LogEvent</code> from an associated logger.
	     *  A target uses this method to translate the event into the appropriate
	     *  format for transmission, storage, or display.
	     *  This method will be called only if the event's level is in range of the
	     *  target's level.
	     *
	     *  <b><i>Descendants need to override this method to make it useful.</i></b>
	     */
	    public override function logEvent (event:LogEvent):void
	    {
	    	trace (event);
	    	var level:int = event.level;
	    	var targetLevel:int;
	    	switch (level)
	    	{
	    		case LogEventLevel.DEBUG:
	    		{
	    			targetLevel = LEVEL_DEBUG;
	    			break;
	    		}
	    		case LogEventLevel.ERROR:
	    		{
	    			targetLevel = LEVEL_ERROR;
	    			break;
	    		}
	    		case LogEventLevel.FATAL:
	    		{
	    			targetLevel = LEVEL_ERROR;
	    			break;
	    		}
	    		case LogEventLevel.INFO:
	    		{
	    			targetLevel = LEVEL_INFORMATION;
	    			break;
	    		}
	    		case LogEventLevel.WARN:
	    		{
	    			targetLevel = LEVEL_WARNING;
	    			break;
	    		}
	    		default:
	    		{
	    			targetLevel = LEVEL_ALL;
	    			break;
	    		}
	    	}
	    		
    		trace (aConnection);
    		
			if (aConnection == null)
			{
				trace ("creating connection");
				aConnection = new LocalConnection();
				aConnection.addEventListener (StatusEvent.STATUS, onConnectionStatus);
			}
			
	 		var category:String = includeCategory ?
								  ILogger(event.target).category + fieldSeparator :
								  "";
								  
			var message:String = event.message;
			message = (typeof message == "xml") ? 
						(message as XML).toXMLString() : message.toString();
			message = category + message;
			
			aConnection.send("_xpanel1", "dispatchMessage", 
							getTimer(), 
							message, 
							targetLevel);
	    }
	    
	    private function onConnectionStatus (event:StatusEvent):void
	    {
			switch (event.level) 
			{
				case "error":
				{
					dispatchEvent (new Event (FAULT));
					break;
				}
			}
	    }
               
		public function addEventListener(type:String, 
									listener:Function, 
									useCapture:Boolean = false, 
									priority:int = 0, 
									useWeakReference:Boolean = false):void
		{
			dispatcher.addEventListener(type, listener, useCapture, priority);
		}
		   
		public function dispatchEvent(evt:Event):Boolean
		{
			return dispatcher.dispatchEvent(evt);
		}
		
		public function hasEventListener(type:String):Boolean
		{
			return dispatcher.hasEventListener(type);
		}
		
		public function removeEventListener(type:String, 
									listener:Function, 
									useCapture:Boolean = false):void
		{
			dispatcher.removeEventListener(type, listener, useCapture);
		}
		   
		public function willTrigger(type:String):Boolean 
		{
			return dispatcher.willTrigger(type);
		}
	}
}